<?php

/**
 * @file
 * Utility functions for the SCSS compiler.
 */

/**
 * Defines the configuration form.
 *
 * @see system_settings_form()
 *
 * @return array
 *   The renderable form.
 */
function scss_config_form() {

  $form['scss_active'] = [
    '#title' => t('Watch for theme changes and compile SASS on Drupal page load'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('scss_active', FALSE),
  ];

  $options = [];
  foreach (list_themes() as $theme_key => $theme_data) {
    $options[$theme_key] = $theme_data->info['name'];
  }

  $form['scss_theme_to_watch'] = [
    '#title' => t('Theme'),
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => variable_get('scss_theme_to_watch', SCSS_DEFAULT_THEME),
  ];

  $form['scss_theme_directory_to_watch'] = [
    '#title' => t('Directory name containing SASS/SCSS source files'),
    '#type' => 'textfield',
    '#default_value' => variable_get('scss_theme_directory_to_watch', SCSS_DIRECTORY_TO_WATCH),
    '#description' => t('This is usually <i>sass</i> or <i>scss</i>and is relative to theme root.'),
  ];

  $form['scss_css_directory'] = [
    '#title' => t('CSS destination directory'),
    '#type' => 'textfield',
    '#default_value' => variable_get('scss_css_directory', SCSS_DEFAULT_CSS_DIRECTORY),
    '#description' => t('This is usually <i>css</i> and is relative to theme root.'),
  ];

  $theme = variable_get('scss_theme_to_watch', SCSS_DEFAULT_THEME);
  $theme_directory = drupal_get_path('theme', $theme);

  $form['scss_additional_import_paths'] = [
    '#title' => t('Additional import paths'),
    '#type' => 'textarea',
    '#default_value' => variable_get('scss_additional_import_paths', ''),
    '#description' => t("One path per line. The theme directory path will be prepended to each path. Currently: '@theme_directory/'", ['@theme_directory' => $theme_directory]),
  ];

  $options = [];
  foreach (['Expanded', 'Nested', 'Compressed', 'Compact', 'Crunched'] as $value) {
    $options['ScssPhp\ScssPhp\Formatter\\' . $value] = $value;
  }

  $form['scss_output_formatting'] = [
    '#title' => t('Output formatting'),
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => variable_get('scss_output_formatting', SCSS_DEFAULT_OUTPUT_FORMAT),
  ];

  $form['scss_line_numbers'] = [
    '#title' => t('Enable line numbers'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('scss_line_numbers', FALSE),
  ];

  $form['scss_source_maps'] = [
    '#title' => t('Enable source maps'),
    '#type' => 'checkbox',
    '#default_value' => variable_get('scss_source_maps', TRUE),
  ];

  $form['scss_force_compile'] = [
    '#value' => t('Compile and write'),
    '#type' => 'button',
    '#name' => 'compile',
    '#weight' => 1000,
  ];

  $form['scss_test_compile'] = [
    '#value' => t('Test'),
    '#type' => 'button',
    '#name' => 'test',
    '#weight' => 1000,
  ];

  return system_settings_form($form);
}

/**
 * Implements hook_validate().
 *
 * {@inheritDoc}
 */
function scss_config_form_validate($form, &$form_state) {

  // Do not validate if resetting.
  if ($form_state['clicked_button']['#value'] == t('Reset to defaults')) {
    return;
  }

  // Force compile button - skip rest of form with drupal_goto.
  if ($form_state['clicked_button']['#name'] == 'compile') {
    scss_compile();
    drupal_goto($_GET['q']);
  }

  // Test compile button - skip rest of form with drupal_goto.
  if ($form_state['clicked_button']['#name'] == 'test') {
    scss_compile(TRUE);
    drupal_goto($_GET['q']);
  }

  // Are we switching themes?
  $theme = $form_state['values']['scss_theme_to_watch'];
  $theme_directory = drupal_get_path('theme', $theme);
  if ($theme != variable_get('scss_theme_to_watch', SCSS_DEFAULT_THEME)) {

    // Check for rb file; use data if exists.
    $rb_file_path = "$theme_directory/config.rb";
    if (file_exists($rb_file_path)) {

      // Get config.rb information and parse.
      $explode = explode("\n", file_get_contents($rb_file_path));
      // Strip out comments.
      foreach ($explode as $key => $line) {
        if (strpos($line, '#') !== FALSE) {
          unset($explode[$key]);
        }
      }

      // Parse into data.
      $data = drupal_parse_info_format(implode("\n", $explode));

      $updated = FALSE;
      // Set scss_theme_directory_to_watch.
      if (!empty($data['sass_dir'])) {
        $form_state['values']['scss_theme_directory_to_watch'] = $data['sass_dir'];
        $updated = TRUE;
      }

      // Set scss_theme_directory_to_watch.
      if (!empty($data['css_dir'])) {
        $form_state['values']['scss_css_directory'] = $data['css_dir'];
        $updated = TRUE;
      }

      // Set scss_theme_directory_to_watch.
      if (!empty($data['additional_import_paths'])) {
        $form_state['values']['scss_additional_import_paths'] = '';
        foreach (explode(',', $data['additional_import_paths']) as $value) {
          $value = str_replace(['[', '"', ']'], '', $value);
          $form_state['values']['scss_additional_import_paths'] .= trim($value) . "\n";
        }

        $updated = TRUE;
      }

      if ($updated) {
        drupal_set_message(t('Values from @rb_file have been used to populate settings', ['@rb_file' => $rb_file_path]));
      }

    }
  }

  // Check to see if library will load before activating compile function.
  if ($form_state['values']['scss_active']) {
    $library = libraries_load('scssphp');
    if (empty($library['loaded'])) {
      form_set_error('scss_active', 'Error loading scssphp library. Cannot activate SCSS/SASS change detection.');
    }
  }

  // Check for source SASS file and directory.
  $sass_directory = $form_state['values']['scss_theme_directory_to_watch'];
  $sass_directory_full = "{$theme_directory}/{$sass_directory}";

  // Check if directory exists.
  if (!file_exists($sass_directory_full)) {
    $message = t('Directory does not exist at @dir', ['@dir' => $sass_directory_full]);
    drupal_set_message($message, 'warning');
  }

  // Get list of available sass files.
  $files = scss_get_files($sass_directory_full);

  // Check for any files.
  if (empty($files)) {
    drupal_set_message(t('No SCSS/SASS files detected at: @dir', ['@dir' => $sass_directory_full]), 'warning');
  }
  elseif ($form_state['values']['scss_active']) {
    drupal_set_message(t('Watching SCSS/SASS files: @list', ['@list' => implode(', ', $files)]));
  }

  // Validating - destination writable.
  $css_directory = "{$theme_directory}/{$form_state['values']['scss_css_directory']}";
  if (!is_writable($css_directory)) {
    drupal_set_message(t('Destination not writable @dir', ['@dir' => $css_directory]), 'warning');
  }

}
